
#include "RGB_LED_Matrix_GCC.h"

void RGB_LED_MATRIX_start(int rxpin, int txpin, int rxtxmode, int baud_rate)
{
    /*start Serial port, default baudrate 19200bps*/
    serial_driver_llstart(rxpin, txpin, rxtxmode, baud_rate);
    waitcnt(CLKFREQ/2 + CNT);
}


void RGB_LED_MATRIX_tx_data_packet(char command, long value_32bit)
{    
    serial_tx('P');
    serial_tx('M');
    serial_tx('T');
    serial_tx('X');
    serial_tx(command);
    serial_tx((char) (value_32bit >> 24));
    serial_tx((char) (value_32bit >> 16));
    serial_tx((char) (value_32bit >> 8));
    serial_tx((char) value_32bit);
}


void RGB_LED_MATRIX_tx_color_data(char *buffer, long writebytes)
{
    while((writebytes--) > 0)
    {
        serial_tx((char) *buffer);
        buffer++;
    }
}

void RGB_LED_MATRIX_tx_data_length(long data_length)
{
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_SEND_DATA, data_length);
}

void RGB_LED_MATRIX_set_window(long x_start, long y_start, long x_end, long y_end)
{
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_SET_WINDOW_BOUND_START_X, x_start);
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_SET_WINDOW_BOUND_START_Y, y_start);
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_SET_WINDOW_BOUND_END_X, x_end);
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_SET_WINDOW_BOUND_END_Y, y_end);
}

void RGB_LED_MATRIX_set_pointer(long x, long y)
{    
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_SET_POINTER_X, x);
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_SET_POINTER_Y, y);
}

void RGB_LED_MATRIX_screen_inverse(long gdram_x)
{
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_SCREEN_COLOR_INVERSE, gdram_x);
}
void RGB_LED_MATRIX_screen_overlay(long gdram_x)
{
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_SCREEN_OVERLAY, gdram_x);
}

void RGB_LED_MATRIX_screen_minuslay(long gdram_x)
{
     RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_SCREEN_MINUSLAY, gdram_x);
}

void RGB_LED_MATRIX_screen_gray(long gdram_x)
{
     RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_SCREEN_COLOR_GRAY, gdram_x);
}

void RGB_LED_MATRIX_save_picture_eeporm(long pic_num)
{    
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_SAVE_PICTURE_EEPROM,pic_num);
}

void RGB_LED_MATRIX_load_picture_eeporm(long pic_num)
{    
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_LOAD_PICTURE_EEPROM,pic_num);
}

void RGB_LED_MATRIX_display_parallax_logo_nbg(long x, long y)
{
    RGB_LED_MATRIX_set_pointer(x, y);
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_DISPLAY_PARALLAX_LOGO_NBG, 0x000000);
}


void RGB_LED_MATRIX_display_parallax_logo(long x, long y, long background_color)
{
    RGB_LED_MATRIX_set_pointer(x, y);
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_DISPLAY_PARALLAX_LOGO, background_color);
}


void RGB_LED_MATRIX_display_propeller_hat_nbg(long x, long y)
{
    RGB_LED_MATRIX_set_pointer(x, y);
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_DISPLAY_PROPELLER_HAT_NBG, 0x000000);
}

void RGB_LED_MATRIX_display_propeller_hat(long x, long y)
{
    RGB_LED_MATRIX_set_pointer(x, y);
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_DISPLAY_PROPELLER_HAT, 0x000000);

}

void RGB_LED_MATRIX_display_customer_bmp1(long x, long y)
{
    RGB_LED_MATRIX_set_pointer(x, y);
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_DISPLAY_CUSTOMER_BMP1, 0x000000);
}

void RGB_LED_MATRIX_display_customer_bmp2(long x, long y)
{
    RGB_LED_MATRIX_set_pointer(x, y);
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_DISPLAY_CUSTOMER_BMP2, 0x000000);
}

void RGB_LED_MATRIX_display_bxGDRAM(long bxGDRAM)
{
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_DISPLAY_BXGDRAM, bxGDRAM);
}

void RGB_LED_MATRIX_copy_GDRAM_to(long bxGDRAM)
{
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_COPY_2_BXGDRAM, bxGDRAM);
}

void RGB_LED_MATRIX_clear_screen(long ccolor)
{
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_FILL_GDRAM, ccolor);
}

void RGB_LED_MATRIX_clear_ascii_nbg(void)
{
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_DISPLAY_BXGDRAM, 0x000000);
}


void RGB_LED_MATRIX_display_ascii_1632_nbg(char c, long start_x, long start_y, long foreground_color)
{
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_SET_FOREGROUND_COLOR, foreground_color);
    RGB_LED_MATRIX_set_pointer(start_x, start_y);
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_DISPLAY_CHAR_1632_NBG, c);
}



void RGB_LED_MATRIX_display_ascii_1632(char c, long start_x, long start_y, long foreground_color, long background_color)
{
    RGB_LED_MATRIX_set_colors(foreground_color, background_color);
    RGB_LED_MATRIX_set_pointer(start_x, start_y);
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_DISPLAY_CHAR_1632, c);
}

void RGB_LED_MATRIX_display_ascii_0816_nbg(char c, long start_x, long start_y, long foreground_color)
{
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_SET_FOREGROUND_COLOR, foreground_color);
    RGB_LED_MATRIX_set_pointer(start_x, start_y);
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_DISPLAY_CHAR_0816_NBG, c);
}


void RGB_LED_MATRIX_display_ascii_0816(char c, long start_x, long start_y, long foreground_color, long background_color)
{
    RGB_LED_MATRIX_set_colors(foreground_color, background_color);
    RGB_LED_MATRIX_set_pointer(start_x, start_y);
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_DISPLAY_CHAR_0816, c);
}


void RGB_LED_MATRIX_display_ascii_0508_nbg(char c, long start_x, long start_y, long foreground_color)
{
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_SET_FOREGROUND_COLOR, foreground_color);
    RGB_LED_MATRIX_set_pointer(start_x, start_y);
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_DISPLAY_CHAR_0508_NBG, c);
}

void RGB_LED_MATRIX_display_ascii_0508(char c, long start_x, long start_y, long foreground_color, long background_color)
{
    RGB_LED_MATRIX_set_colors(foreground_color, background_color);
    RGB_LED_MATRIX_set_pointer(start_x, start_y);
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_DISPLAY_CHAR_0508, c);
}

void RGB_LED_MATRIX_set_colors(long foreground_color, long background_color)
{
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_SET_FOREGROUND_COLOR, foreground_color);
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_SET_BACKGROUND_COLOR, background_color);

}


void RGB_LED_MATRIX_circle(long x, long y, long r, long circle_color)
{
    RGB_LED_MATRIX_set_pointer(x, y);
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_SET_RADIUS, r);
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_DRAW_CIRCLE, circle_color);
}

void RGB_LED_MATRIX_line(long x0, long y0, long x1, long y1, long line_color)
{
    RGB_LED_MATRIX_set_pointer(x0, y0);    
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_SET_LINE_END_X, x1);
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_SET_LINE_END_Y, y1);
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_DRAW_LINE, line_color);
}

void RGB_LED_MATRIX_dot(long x, long y, long dot_color)
{

    RGB_LED_MATRIX_set_pointer(x, y);    
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_DRAW_DOT, dot_color);
}


void RGB_LED_MATRIX_reset(void)
{
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_RESET, 0x00000000);
}

void RGB_LED_MATRIX_update_screen(void)
{
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_UPDATE_SCREEN, 0x00000000);
}


/********************************************************



*********************************************************/
void RGB_LED_MATRIX_change_screen_size(long new_pseudo_screen_size)
{
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_CHANGE_SCREEN_SIZE, new_pseudo_screen_size);
}


void RGB_LED_MATRIX_change_device_id(long new_id)
{
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_CHANGE_DEVICEID, new_id);
}

void RGB_LED_MATRIX_change_baudrate(long new_baudrate)
{
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_SET_BAUDRATE, new_baudrate);
}


void RGB_LED_MATRIX_get_system_configuration(char * read_buffer)
{
    long index = 0;
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_GET_SYSTEM_CONFIGURATION, 0x00000000);
    while(index < 16)// One less than the size of the array
    {
        *read_buffer = serial_rxtime(2); // Read a character
        read_buffer++;
        index++;
    }
}

void RGB_LED_MATRIX_get_device_id(char * read_buffer)
{
    long index = 0;
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_GET_DEVICEID, 0x00000000);
    while(index < 16)// One less than the size of the array
    {
        *read_buffer = serial_rxtime(2); // Read a character
        read_buffer++;
        index++;
    }
}

void RGB_LED_MATRIX_get_screen_size(char * read_buffer)
{
    long index = 0;
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_GET_SCREEN_SIZE, 0x00000000);
    while(index < 16)// One less than the size of the array
    {
        *read_buffer = serial_rxtime(2); // Read a character
        read_buffer++;
        index++;
    }
}
  
