/*
*************************************************************************************
**
**                             
**
** Files      : RGB_LED_MatrixGCC.h
**
** Version : 1.1
**
** Date     : 2012/05/10
**
** Description:
**
**
** History:
**    1. Date: 2012/05/10
**       Author: John Zhong
**       Modification: Create
** 
**    2. Date: 2012/11/09
**       Author: John Zhong
**       Modification: Update the API
**
***************************************************************************************
*/


#ifndef RGB_LED_MATRIX_GCC_H
#define RGB_LED_MATRIX_GCC_H

#include <propeller.h>
#include <cog.h>
#include "Serial_Driver_Xbuff_Low_Level.h"


#define RGB_LED_MATRIX_MODE_NORMAL  (0)
#define RGB_LED_MATRIX_MODE_INVERSE      (1)
#define RGB_LED_MATRIX_MODE_OVERLAY     (2)
#define RGB_LED_MATRIX_MODE_MINUSLAY   (3)
#define RGB_LED_MATRIX_MODE_GRAY    (4)
#define RGB_LED_MATRIX_MODES (RGB_LED_MATRIX_MODE_GRAY + 1) // How many Modes the module can support 





#define    RGB_LED_MATRIX_UPDATE_SCREEN                   (0x00)

#define    RGB_LED_MATRIX_SET_POINTER_X                   (0x03)
#define    RGB_LED_MATRIX_SET_POINTER_Y                   (0x04)
#define    RGB_LED_MATRIX_SEND_DATA                      (0x05)
#define    RGB_LED_MATRIX_SET_WINDOW_BOUND_START_X         (0x06)           // NOTICE: AFTER SET THE WINDOWS BOUND, THE POINTERX AND POINTERY
#define    RGB_LED_MATRIX_SET_WINDOW_BOUND_START_Y         (0x07)           //         WILL POINT TO WINDOWS BOUND START X AND START Y AUTOMATIC
#define    RGB_LED_MATRIX_SET_WINDOW_BOUND_END_X           (0x08)           //
#define    RGB_LED_MATRIX_SET_WINDOW_BOUND_END_Y           (0x09)          //
#define    RGB_LED_MATRIX_COPY_2_BXGDRAM                  (0x0A)
#define    RGB_LED_MATRIX_DISPLAY_BXGDRAM                (0x0B)
#define    RGB_LED_MATRIX_FILL_GDRAM                     (0x0C)         // FILL THE GDRAM WITH DATA(USER DEFINED) TXCMD(#define    RGB_LED_MATRIX_FILLGDRAM, (0x01)
/*
#define    RGB_LED_MATRIX_ROLLL_EFT                      (0x0D)
#define    RGB_LED_MATRIX_ROLL_RIGHT                     (0x0E)
#define    RGB_LED_MATRIX_ROLL_UP                        (0x0F)
#define    RGB_LED_MATRIX_ROLL_DOWN                      (0x10)
*/

#define    RGB_LED_MATRIX_SET_FOREGROUND_COLOR            (0x11)
#define    RGB_LED_MATRIX_SET_BACKGROUND_COLOR            (0x12)
#define    RGB_LED_MATRIX_DISPLAY_CHAR_0508               (0x13)
#define    RGB_LED_MATRIX_DISPLAY_CHAR_0816               (0x14)

#define    RGB_LED_MATRIX_DISPLAY_CHAR_0508_NBG            (0x15)             // NBG STAND FOR NO BACKGROUND
#define    RGB_LED_MATRIX_DISPLAY_CHAR_0816_NBG            (0x16)              // IF YOU WANT TO CLEAR THESE CHARACTER IN THE MATRIX PLEASE USE THE COMMAND #define    RGB_LED_MATRIX_DISPLAYBXGDRAM(  ) ABOVE
#define    RGB_LED_MATRIX_DISPLAY_PROPELLER_HAT           (0x17)
#define    RGB_LED_MATRIX_DISPLAY_PROPELLER_HAT_NBG        (0x18)
#define    RGB_LED_MATRIX_DISPLAY_PARALLAX_LOGO           (0x19)
#define    RGB_LED_MATRIX_DISPLAY_PARALLAX_LOGO_NBG        (0x1A)
#define    RGB_LED_MATRIX_DISPLAY_CUSTOMER_BMP1           (0x1B)
#define    RGB_LED_MATRIX_DISPLAY_CUSTOMER_BMP2           (0x1C)
#define    RGB_LED_MATRIX_DRAW_DOT                       (0x1D)

#define    RGB_LED_MATRIX_SET_LINE_END_X                   (0x1E)
#define    RGB_LED_MATRIX_SET_LINE_END_Y                   (0x1F)
#define    RGB_LED_MATRIX_DRAW_LINE                      (0x20)

#define    RGB_LED_MATRIX_SET_RADIUS                     (0x21)
#define    RGB_LED_MATRIX_DRAW_CIRCLE                    (0x22)

#define    RGB_LED_MATRIX_SAVE_PICTURE_EEPROM           (0x23)
#define    RGB_LED_MATRIX_LOAD_PICTURE_EEPROM           (0x24)
#define    RGB_LED_MATRIX_FILL_CIRCLE                    (0x25)


#define    RGB_LED_MATRIX_SCREEN_COLOR_INVERSE            (0x30)
#define    RGB_LED_MATRIX_SCREEN_OVERLAY                 (0x31)
#define    RGB_LED_MATRIX_SCREEN_MINUSLAY                (0x32)
#define    RGB_LED_MATRIX_SCREEN_COLOR_GRAY               (0x33)

#define    RGB_LED_MATRIX_DISPLAY_CHAR_1632               (0x40)
#define    RGB_LED_MATRIX_DISPLAY_CHAR_1632_NBG           (0x41)

#define    RGB_LED_MATRIX_RESET                         (0xA0)
#define    RGB_LED_MATRIX_SET_BAUDRATE                   (0xA1)
#define    RGB_LED_MATRIX_SET_MODE                       (0xA2)
#define    RGB_LED_MATRIX_GET_VERSION                 (0xA6)
#define    RGB_LED_MATRIX_GET_SCREEN_SIZE                 (0xA7)
#define    RGB_LED_MATRIX_CHANGE_SCREEN_SIZE              (0xA8)
#define    RGB_LED_MATRIX_GET_DEVICEID                   (0xA9)
#define    RGB_LED_MATRIX_CHANGE_DEVICEID                (0xAA)
#define    RGB_LED_MATRIX_GET_SYSTEM_CONFIGURATION        (0xAB)

typedef unsigned char uint_8;
typedef unsigned short uint_16;
typedef unsigned long uint_32;



void RGB_LED_MATRIX_start(int rxpin, int txpin, int rxtxmode, int baud_rate);
void RGB_LED_MATRIX_tx_data_packet(char command, long value_32bit);
void RGB_LED_MATRIX_tx_color_data(char *buffer, long writebytes);
void RGB_LED_MATRIX_tx_data_length(long data_length);
void RGB_LED_MATRIX_set_window(long x_start, long y_start, long x_end, long y_end);
void RGB_LED_MATRIX_set_pointer(long x, long y);
void RGB_LED_MATRIX_screen_inverse(long gdram_x);
void RGB_LED_MATRIX_screen_overlay(long gdram_x);
void RGB_LED_MATRIX_screen_minuslay(long gdram_x);
void RGB_LED_MATRIX_screen_gray(long gdram_x);
void RGB_LED_MATRIX_save_picture_eeporm(long pic_num);
void RGB_LED_MATRIX_load_picture_eeporm(long pic_num);
void RGB_LED_MATRIX_display_parallax_logo_nbg(long x, long y);
void RGB_LED_MATRIX_display_parallax_logo(long x, long y, long background_color);
void RGB_LED_MATRIX_display_propeller_hat_nbg(long x, long y);
void RGB_LED_MATRIX_display_propeller_hat(long x, long y);
void RGB_LED_MATRIX_display_customer_bmp1(long x, long y);
void RGB_LED_MATRIX_display_customer_bmp2(long x, long y);
void RGB_LED_MATRIX_display_bxGDRAM(long bxGDRAM);
void RGB_LED_MATRIX_copy_GDRAM_to(long bxGDRAM);
void RGB_LED_MATRIX_clear_screen(long ccolor);
void RGB_LED_MATRIX_clear_ascii_nbg(void);
void RGB_LED_MATRIX_display_ascii_1632_nbg(char c, long start_x, long start_y, long foreground_color);
void RGB_LED_MATRIX_display_ascii_1632(char c, long start_x, long start_y, long foreground_color, long background_color);
void RGB_LED_MATRIX_display_ascii_0816_nbg(char c, long start_x, long start_y, long foreground_color);
void RGB_LED_MATRIX_display_ascii_0816(char c, long start_x, long start_y, long foreground_color, long background_color);
void RGB_LED_MATRIX_display_ascii_0508_nbg(char c, long start_x, long start_y, long foreground_color);
void RGB_LED_MATRIX_display_ascii_0508(char c, long start_x, long start_y, long foreground_color, long background_color);
void RGB_LED_MATRIX_set_colors(long foreground_color, long background_color);
void RGB_LED_MATRIX_circle(long x, long y, long r, long circle_color);
void RGB_LED_MATRIX_line(long x0, long y0, long x1, long y1, long line_color);
void RGB_LED_MATRIX_dot(long x, long y, long dot_color);
void RGB_LED_MATRIX_reset(void);
void RGB_LED_MATRIX_update_screen(void);
/********************************************************
*********************************************************/
void RGB_LED_MATRIX_change_screen_size(long new_pseudo_screen_size);
void RGB_LED_MATRIX_change_device_id(long new_id);
void RGB_LED_MATRIX_change_baudrate(long new_baudrate);
void RGB_LED_MATRIX_get_system_configuration(char * read_buffer);
void RGB_LED_MATRIX_get_device_id(char * read_buffer);
void RGB_LED_MATRIX_get_screen_size(char * read_buffer);

#endif //RGB_LED_MATRIX_GCC_H