/*
File: RGB_LED_MATRIX_Simple_Test.c  
*/
#include <propeller.h>
#include "RGB_LED_MATRIX_GCC.h"

#define  RGB_LED_MATRIX_SIO (0) /* connected the module to pin 0*/
#define  RGB_LED_MATRIX_COLOR_BLACK (0x00000000)
#define  RGB_LED_MATRIX_COLOR_WHITE (0x00FFFFFF)
#define  RGB_LED_MATRIX_COLOR_RED (0x000000FF)
#define  RGB_LED_MATRIX_COLOR_GREEN (0x0000FF00)
#define  RGB_LED_MATRIX_COLOR_BLUE (0x00FF0000)

/* Rainbow color pattern */
HUBDATA long sixteen_colors_table[16] = 
{
    0x000000FF, 
    0x006500FF, 
    0x00E000FF,
    0x00D600FF,
      
    0x00E600FF,    
    0x00FF00D7, 
    0x00FF0010, 
    0x00FF3C00,
    
    0x00FFB500,    
    0x00F7FF00, 
    0x009EFF00, 
    0x0000FF11,
    
    0x0000FF76,     
    0x0000FFE2, 
    0x0000FDFF, 
    0x00005DFF,    
};

/*******************************************************
         Function: display the 0816 ASCII string
*******************************************************/

void string_0816NBG(char * str, long start_x, long start_y, long foreground_color)
{
    int index = 0;
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_SET_FOREGROUND_COLOR, foreground_color);
    RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_SET_POINTER_Y, start_y);
    while(*str != '\0')
    {
        RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_SET_POINTER_X, start_x + (index * 8));
        RGB_LED_MATRIX_tx_data_packet(RGB_LED_MATRIX_DISPLAY_CHAR_0816_NBG, *str++);
        index++;
    }
}


int main(void)
{
    signed int index = 0;    
/*******************************************************
              Start and init the serial port
              Clear the screen
*******************************************************/
    RGB_LED_MATRIX_start(1,RGB_LED_MATRIX_SIO,  0, 19200); /* Initialize the serial port*/
    RGB_LED_MATRIX_clear_screen(RGB_LED_MATRIX_COLOR_BLACK); /* clear the screen */
    RGB_LED_MATRIX_update_screen();    
 
    while(1)
    {
/*******************************************************
              Display 'S', 'T'
*******************************************************/
        RGB_LED_MATRIX_display_ascii_0508_nbg('S', 0, 0,RGB_LED_MATRIX_COLOR_RED);
        RGB_LED_MATRIX_display_ascii_0508_nbg('T', 3, 3,RGB_LED_MATRIX_COLOR_GREEN);
        RGB_LED_MATRIX_update_screen();    
        waitcnt(CLKFREQ + CNT); 
        
/*******************************************************
              Display "SimplyTronics" String 0816
*******************************************************/ 
        for(index = 0; index > -80; index--)
        {
            string_0816NBG("SimplyTronics", index, 0, 0x00FF0000);
            RGB_LED_MATRIX_update_screen();
            RGB_LED_MATRIX_clear_screen(RGB_LED_MATRIX_COLOR_BLACK); /* clear the screen */          
            waitcnt(CLKFREQ/5 + CNT);
        }
        
/*******************************************************
              Display "Rainbow" Pattern 1
*******************************************************/  
        for(index = 15; index >0; index--)
        {
            RGB_LED_MATRIX_line(index, 15, 15, index, sixteen_colors_table[15 - index]);
            waitcnt(CLKFREQ/100 + CNT);  
        }  
        RGB_LED_MATRIX_update_screen();     
        waitcnt(CLKFREQ + CNT); 
        
/*******************************************************
              Display "Rainbow" Pattern 2
*******************************************************/ 
        for(index = 0; index < 16; index++)
        {
            RGB_LED_MATRIX_line(0, index, index, 0, sixteen_colors_table[index]);
            waitcnt(CLKFREQ/100 + CNT);  
        }    
        RGB_LED_MATRIX_update_screen(); 
        waitcnt(CLKFREQ + CNT);  
        
/*******************************************************
              Display "Rainbow" Pattern 3
*******************************************************/  
        for(index = 0; index < 16; index++)
        {
            RGB_LED_MATRIX_line(index, 0, index, 15, sixteen_colors_table[index]);
            waitcnt(CLKFREQ/100 + CNT);  
        }
        RGB_LED_MATRIX_update_screen();
        waitcnt(CLKFREQ + CNT); 
        
        
/*******************************************************
              Display "Rainbow" Pattern 4
*******************************************************/
          for(index = 2; index < 8; index++)
          {
              RGB_LED_MATRIX_circle(7, 7, index, sixteen_colors_table[index<<1]);
              waitcnt(CLKFREQ/100 + CNT);  
          } 
           RGB_LED_MATRIX_update_screen();             
          waitcnt(CLKFREQ + CNT);  
          
/*******************************************************
              Display "Rainbow" Pattern 5
*******************************************************/
        RGB_LED_MATRIX_copy_GDRAM_to(0);
        for(index = 2; index < 8; index++)
        {
            RGB_LED_MATRIX_circle(7, 7, index, sixteen_colors_table[index]);
            RGB_LED_MATRIX_update_screen();
            waitcnt(CLKFREQ/5 + CNT);
            RGB_LED_MATRIX_display_bxGDRAM(0);
            RGB_LED_MATRIX_update_screen();        
        }
/*******************************************************
              Display default BMP file 
Notice: You need to update the firmware with your BMP files,
              before using the display customer BMP2
*******************************************************/
        RGB_LED_MATRIX_display_customer_bmp1(0, 0);    
        RGB_LED_MATRIX_update_screen();        
    
        waitcnt(CLKFREQ*2 + CNT);
        
/*******************************************************
              Screen inverse
*******************************************************/
        RGB_LED_MATRIX_copy_GDRAM_to(0);
        RGB_LED_MATRIX_screen_inverse(0);
        RGB_LED_MATRIX_update_screen();    
        waitcnt(CLKFREQ + CNT);
        
/*******************************************************
              Restore the screen
*******************************************************/        
        
        RGB_LED_MATRIX_display_bxGDRAM(0);
        RGB_LED_MATRIX_update_screen();    
        waitcnt(CLKFREQ + CNT);
        
/*******************************************************
          Store the screen to bxGDRAM 1 for future use
*******************************************************/

        RGB_LED_MATRIX_copy_GDRAM_to(1);

/*******************************************************
          Display 0508 ASCII without background color
*******************************************************/
        for(index = '1'; index < 'J'; index++)
        {
            RGB_LED_MATRIX_display_ascii_0508_nbg(index, index%8, index%8, 0x00FFFFFF);           
            RGB_LED_MATRIX_update_screen();    
            waitcnt(CLKFREQ/10 + CNT);
            RGB_LED_MATRIX_display_bxGDRAM(0);
        }
        
/*******************************************************
          Display 0816 ASCII without background color
*******************************************************/
        for(index = '1'; index < 'J'; index++)
        {
            RGB_LED_MATRIX_display_ascii_0816_nbg(index, index%8, 0, 0x00FFFFFF);           
            RGB_LED_MATRIX_update_screen();    
            waitcnt(CLKFREQ/10 + CNT);
            RGB_LED_MATRIX_display_bxGDRAM(0);
        }   
        

/*******************************************************
          Display 0508 ASCII with background color
*******************************************************/   
        for(index = '1'; index < 'J'; index++)
        {
            RGB_LED_MATRIX_display_ascii_0508(index, index%8, index%8, sixteen_colors_table[index%16], 0x000F0000);           
            RGB_LED_MATRIX_update_screen();    
            waitcnt(CLKFREQ/10 + CNT);
            RGB_LED_MATRIX_clear_screen(RGB_LED_MATRIX_COLOR_BLACK); 
        }
/*******************************************************
          Display 00816 ASCII with background color
*******************************************************/  
        for(index = '1'; index < 'J'; index++)
        {
            RGB_LED_MATRIX_display_ascii_0816(index, index%8, 0, sixteen_colors_table[index%16], 0x000F0000);           
            RGB_LED_MATRIX_update_screen();    
            waitcnt(CLKFREQ/10 + CNT);
            RGB_LED_MATRIX_clear_screen(RGB_LED_MATRIX_COLOR_BLACK); 
        }   
    
/*******************************************************
              Restore the screen
*******************************************************/
  
        RGB_LED_MATRIX_display_bxGDRAM(1);
        RGB_LED_MATRIX_update_screen();    
        waitcnt(CLKFREQ/5 + CNT);
        
/*******************************************************
 Display built in Parallax LOGO without background color
*******************************************************/
        RGB_LED_MATRIX_set_colors(0x00FF00FF, 0x000000);
        RGB_LED_MATRIX_copy_GDRAM_to(0);
        for(index = 0; index > -72; index--)
        {
            RGB_LED_MATRIX_display_bxGDRAM(0);
            RGB_LED_MATRIX_display_parallax_logo_nbg(index, 0);
            RGB_LED_MATRIX_update_screen();  
            waitcnt(CLKFREQ/10 + CNT);  
        }
        
/*******************************************************
   Display built in Parallax LOGO with background color
*******************************************************/
        RGB_LED_MATRIX_set_colors(0x00FFFFFF, 0x005A0000);  
        for(index = 0; index > -72; index--)
        {
            RGB_LED_MATRIX_display_parallax_logo(index, 0, 0x005A0000);
            RGB_LED_MATRIX_update_screen();  
            waitcnt(CLKFREQ/10 + CNT);
            RGB_LED_MATRIX_clear_screen(RGB_LED_MATRIX_COLOR_BLACK);       
        }
        
            
    }
    return 0;
}        